// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

/* Defines PI with float precision if this is not already defined: */
#ifndef M_PI
#define M_PI          3.14159265f
#endif

/* Defines the threshold to which the difference between two components should be considered as equal: */
#define COMPARE_DELTA 0.0001f

/* Includes the some math functions like abs(...), max(...), etc.: */
#include <cmath>
#include <cfloat>

/**
 * This class should define a 3D vector in homogeneous coordinates.
 *
 * Note:
 * Homogeneous coordinates means that there are the three typical
 * components x,y and z
 *
 * You will learn more about this kind of coordinates in the lecture.
 * For now it is sufficient to know that for homogeneous coordinates
 * always the following applies:
 *
 */

class Vec3f
{
public:
    /** The x component of the vector */
    float x;

    /** The y component of the vector */
    float y;

    /** The z component of the vector */
    float z;

    /**
     * Constructor which creates a *point* at (0,0,0).
     */
    Vec3f();

    /**
     * Constructor which creates a *point* with the given parameters (x, y, z).
     */
    Vec3f(float x, float y, float z);

    /**
     * Constructor which creates a Vec3 with (data[0], data[1], data[2]).
     */
    Vec3f(float data[3]);

    /**
     * Returns the length of this vector.
     */
    float length() const;

    /**
     * Returns the squared length of this vector.
     */
    float squaredLength() const;

    /**
     * Return the Euclidean distance from this point to the given point.
     *
     * NOTE: Since we use this function only with points, only use the
     * (x,y,z) coordinate to calculate the distance
     */
    float distanceTo(const Vec3f points) const;

    /**
     * Returns the dot product of this vector with the given vector
     *
     * NOTE: Use only x,y and z coordinates
     */
    float dot(const Vec3f vector) const;

    /**
     * Returns the cross product of this vector with the given vector
     *
     * NOTE: Use only x,y and z coordinates
     */
    Vec3f cross(const Vec3f vector) const;

    /**
     * Returns a normalized Vector of this vector, so that the length
     * is 1. (Note that normalization only makes sense for vectors, not
     * for points, so again you can assume that w=0).
     */

    Vec3f normalized() const;

    /**
     * Returns a new Vec3f with the x, z of the original
     * Vec3f, but with y = 0.
     */
    Vec3f xz() const;

    /**
     * Returns a new Vec3f with the x, y of the original
     * Vec3f, but with z = 0.
     */
    Vec3f xy() const;

    /**
     * Returns a new Vec3f with the y, z of the original
     * Vec3f, but with x = 0.
     */
    Vec3f yz() const;

    /**
     * Checks that all components are similar enough for both vectors to be
     * considered equal.
     *
     * NOTE: Use 'COMPARE_DELTA' to check whether the difference is small
     * enough.
     */

    bool operator==(const Vec3f vector) const;

    /**
     * Checks if at least one component has a higher difference than
     * 'COMPARE_DELTA' when comparing this vector to the given one.
     */

    bool operator!=(const Vec3f vector) const;

    /**
     * Returns this vector negated.
     */
    Vec3f operator-() const;

    /**
     * Returns a vector where each component of this vector is added by the
     * respective component of the given vector (also the w component!).
     */
    Vec3f operator+(const Vec3f vector) const;

    /**
     * Returns a new vector where each component of this vector is subtracted
     * by the respective component of the given vector (also the w component!).
     */
    Vec3f operator-(const Vec3f vector) const;

    /**
     * Returns a vector where the x, y and z component of this vector are
     * multiplicated by the given scalar (note that w should NOT be scaled!)
     */
    Vec3f operator*(const float scalar) const;

    /**
     * Returns a vector where the x, y and z component of this vector are
     * divided by the given scalar (note that w should NOT be scaled!).
     */
    Vec3f operator/(const float scalar) const;

    /**
     * Allows access to individual components via indices as with arrays,
     * where x is at idx 0, y at idx 1, and so on.
     *
     * The returned value is a reference so that it can be written to it.
     */
    float& operator[](int i);

    /**
     * Returns whether this vector is valid, which means that every
     * component is valid (and not NAN).
     */
    bool valid() const;
};
